;*******************************************************************************
;
;	Filename:		MSMV-1.asm
;	Date:				13 Jun 2016
;	File Version:	1.0
;	Author:			Larry Cicchinelli
;	Description:
;		This program uses a PIC 12F1572 to emulate a 555 timer in its
;				Mono-Stable MultiVibrator mode
;
;		PIC12F1572 I/O:
;			pin 1: VDD
;			pin 2: RA5							Pulse output
;			pin 3: RA4, AN3					Trigger
;			pin 4: ~MCLR
;			pin 5: RA2, AN2
;			pin 6: ICSPCLK
;			pin 7: ISCPDAT
;			pin 8: VSS
;
;
; Definitions used by the program which are associated with specific I/O pins:
;	O_PULSE		- output pulse pin
;	I_TRIGGER	- input trigger pin
;
;
;*******************************************************************************
;*******************************************************************************

		#include p12F1572.inc

; CONFIG1
; __config 0xF9E4
 __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_ON & _CP_OFF & _BOREN_OFF & _CLKOUTEN_OFF
; CONFIG2
; __config 0xFEFF
 __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_OFF & _STVREN_ON & _BORV_LO & _LPBOREN_OFF & _LVP_ON

I_TRIGGER	equ	RA4
O_PULSE		equ	RA5

TRISA_VAL	equ	~(1<<O_PULSE)

EDGE_REG equ	IOCAP						; IOCAP = trigger on positive edge of input
												; IOCAN = trigger on negative edge of input

;*******************************************************************************
; Vectors
;*******************************************************************************

	org	0
	goto	Main

	org	0x04								; interrupt vector
ISR.Exit
	retfie


;*******************************************************************************
; MAIN PROGRAM
;*******************************************************************************

MAIN_PROG CODE                      ; let linker place main program

Main
	call		init							; initialize the system hardware

; The following is probably the simplest one-shot program using this PIC
; and also generates the shortest pulse.
; The delay from the trigger to the output pulse varies between 3 and 6
; instruction cycles.  With Fcpu = 32 MHz, that is between 375ns and 750ns

; ensure inactive trigger
Main.10
	BANKSEL	IOCAP
	bsf		EDGE_REG, I_TRIGGER		; enable trigger

; wait for active trigger
Main.15
	btfss		IOCAF, I_TRIGGER			; skip next if trigger detected
	goto		Main.15
	bcf		IOCAF, I_TRIGGER			; clear trigger flag

; generate the pulse
	BANKSEL	PORTA
	bsf		LATA, O_PULSE				; pulse on
	bcf		LATA, O_PULSE				; pulse off
	goto		Main.10						; do it all over again

;===============================================================================
;									Initialize Hardware
;===============================================================================

init	; initialize the system hardware

; Oscillator - set to 32MHz using 8MHz clock and PLL
	BANKSEL	OSCCON
	movlw		(1<<SPLLEN) | (b'1110'<<IRCF0) | (b'00'<<SCS0)
	movwf		OSCCON						; enable PLL, IntOsc=8Mhz, clock determined by CONFIG
												; *** SCS must be '00' to allow PLL operation ***
init.10
	btfss		OSCSTAT, PLLR				; skip next if PLL ready
	goto		init.10

; Port A
	BANKSEL	PORTA
	clrf		PORTA
	BANKSEL	TRISA
	movlw		TRISA_VAL
	movwf		TRISA
	BANKSEL	ANSELA
	clrf		ANSELA						; no analog inputs

; Interrupts
	clrf		INTCON						; clear global interrupt enable
	BANKSEL	PIE1
	clrf		PIE1							; clear peripheral interrupt enables
	BANKSEL	PORTA
	return
; init

	END
