;*******************************************************************************
;
;	Filename:		MSMV-2.asm
;	Date:				15 Jun 2016
;	File Version:	1.0
;	Author:			Larry Cicchinelli
;	Description:
;		This program uses a PIC 12F1572 to emulate a 555 timer in its
;				Mono-Stable MultiVibrator mode
;
; The program has delay functions for:
;		1us
;		X us ( 2 <= X <= 255 )
;		X ms ( 1 <= X <= 255 )
; The functions were written so that their timings are exact - as close as is 
;	possible with the internal oscillator of the PIC.
;
; Options:
;		ENABLE_RETRIGGER
;			= 1 = enable retriggering
;			= 0 = no retrigger
;			IMPORTANT: the output pulse will be an integer multiple of the programmed
;				width value because the retrigger signal is tested at the end of the
;				pulse width timing.
;		EDGE_REG
;			= IOCAP = trigger on positive edge of input
;			= IOCAN = trigger on negative edge of input
;	
;		PIC12F1572 I/O:
;			pin 1: VDD
;			pin 2: RA5							Pulse output
;			pin 3: RA4, AN3					Trigger
;			pin 4: ~MCLR
;			pin 5: RA2, AN2
;			pin 6: ICSPCLK
;			pin 7: ISCPDAT
;			pin 8: VSS
;
;
; Definitions used by the program which are associated with specific I/O pins:
;	O_PULSE		- output pulse pin
;	I_TRIGGER	- input trigger pin
;
;
;*******************************************************************************
;*******************************************************************************

		#include p12F1572.inc

; CONFIG1
; __config 0xF9E4
 __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_ON & _CP_OFF & _BOREN_OFF & _CLKOUTEN_OFF
; CONFIG2
; __config 0xFEFF
 __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_OFF & _STVREN_ON & _BORV_LO & _LPBOREN_OFF & _LVP_ON

I_TRIGGER	equ	RA4
O_PULSE		equ	RA5

TRISA_VAL	equ	~(1<<O_PULSE)

; Common RAM allocations: 16 bytes
Access_RAM		udata_shr	0x70		; 16 bytes starting at address 0x70
MScounter		res	1					; millisecond counter

;===============================================================================
; user options
;===============================================================================
ENABLE_RETRIGGER equ 0					; 0 = no re-trigger
												; 1 = enable re-trigger
EDGE_REG equ	IOCAN						; IOCAP = trigger on positive edge of input
												; IOCAN = trigger on negative edge of input
		
;*******************************************************************************
; Vectors
;*******************************************************************************

	org	0
	goto	Main

	org	0x04								; interrupt vector
ISR.Exit
	retfie


;*******************************************************************************
; MAIN PROGRAM
;*******************************************************************************

MAIN_PROG CODE                      ; let linker place main program

Main
	call		init							; initialize the system hardware

Main.10
	BANKSEL	IOCAP
	bsf		EDGE_REG, I_TRIGGER		; enable trigger

; wait for trigger
Main.15
	btfss		IOCAF, I_TRIGGER			; skip next if trigger detected
	goto		Main.15
	bcf		IOCAF, I_TRIGGER			; clear trigger flag

; If Retrigger is NOT enabled, the delay from the trigger to the output pulse
; varies between 6 and 9 instruction cycles.  With Fcpu = 32MHz,  that is
; between 750ns and 1125ns.
;
#if ENABLE_RETRIGGER == 0
	bcf		EDGE_REG, I_TRIGGER		; disable trigger
#endif ; ENABLE_RETRIGGER == 0

Main.20
; generate the pulse - which will be 1 instruction cycle longer than the
; requested delay due to the instructions that turn the pulse on and off
; If ENABLE_RETRIGGER == 1 and no retrigger is detected, the pulse width will
; be 4 instruction cycles longer than requested.
	BANKSEL	PORTA
	bsf		LATA, O_PULSE				; pulse on
	movlw		.10							; delay value
	call		Delay.Wms					; delay for desired pulse width

#if ENABLE_RETRIGGER == 1	
	BANKSEL	IOCAP
	btfss		IOCAF, I_TRIGGER			; skip next if re-trigger detected
	goto		Main.30
	bcf		IOCAF, I_TRIGGER			; clear trigger flag
	goto		Main.20
Main.30
	BANKSEL	PORTA
#endif ; ENABLE_RETRIGGER == 1
	
	bcf		LATA, O_PULSE				; pulse off
	goto		Main.10						; do it all over again
; Main

;===============================================================================
;									Delay Functions
;===============================================================================

; The delay time is dependant on the CPU clock speed.  Most instructions
; execute in four clock cycles.  With a 32MHz clock, the instruction execution
; time is 8MHz which yields an instruction cycle (I-cycle) time of 125ns.


Delay.Wms
; Execute a delay of W milliseconds by executing a specific number of instructions
; The execution of this function requires an additional 6 I-cycles due to the 
; calling sequence and return instruction.  One of them is accounted for by NOT
; having the nop at the end of the loop.  Therefore, the total execution of the
; function, including overhead, is W milliseconds plus 5 I-cycles.
;
; register and memory usege:
;	enter: W = number of ms: 1 <= W <= 255
;	exit: W = 0
; notes:
;	the first number in the comment is the number of I-cycles for the instruction
; The typical calling sequence is:
;		movlw		x
;		call		Delay.Wms
;	which takes 3 I-cycles

	movwf		MScounter					; 1 save the nbr of milliseconds
; the following loop takes exactly 999usec
Delay.Wms.10
	movlw		.250
	call		Delay.Wus
	movlw		.250
	call		Delay.Wus
	movlw		.250
	call		Delay.Wus
	movlw		.249							; set total to 999usec in order to account
	call		Delay.Wus					;	for loop and non-loop instructions
	call		Delay.Wms.20				; 4 for call and return
	decf		MScounter, f				; 1 update counter
	btfss		STATUS, Z					; 1(2) skip next if done
	goto		Delay.Wms.10				; 2
;	nop										; 1 for when MScounter = 0

Delay.Wms.20
	return									; 2
; Delay.Wms


Delay.Wus
; Execute a delay of W microseconds by executing a specific number of instructions.
; register and memory usege:
;	enter: W = number of us: 2 <= W <= 255 (W is not tested for valid value!)
;	exit: W = 0
; notes:
;	the first number in the comment is the number of I-cycles for the instruction

; The typical calling sequence is:
;		movlw		x
;		call		Delay.Wus
;	which takes 3 I-cycles

	decf		WREG, w						; 1 adjust count for function instructions
												; 4 I-cycles up to here
; the following loop uses 8 I-cycles = 1us
;	except the last, when w=0, which takes 7 cycles
Delay.Wus.10
	call		Delay.Wus.20				; 4 call + return
	decf		WREG, w						; 1 update counter
	btfss		STATUS, Z					; 1(2) skip next if W=0
	goto		Delay.Wus.10				; 2
	nop										; 1 neded for last loop iteration
;
	nop										; 1 need to execute a total
	nop										; 1		of 8 I-cycles outside the loop
Delay.Wus.20
	return									; 2
; Delay.Wus


;===============================================================================
;									Initialize Hardware
;===============================================================================

init	; initialize the system hardware

; Oscillator - set to 32MHz using 8MHz clock and PLL
	BANKSEL	OSCCON
	movlw		(1<<SPLLEN) | (b'1110'<<IRCF0) | (b'00'<<SCS0)
	movwf		OSCCON						; enable PLL, IntOsc=8Mhz, clock determined by CONFIG
												; *** SCS must be '00' to allow PLL operation ***
	btfss		OSCSTAT, HFIOFL			; skip next if HF osc ready
	goto		init.10
init.10
	btfss		OSCSTAT, PLLR				; skip next if PLL ready
	goto		init.10

; Port A
	BANKSEL	PORTA
	clrf		PORTA
	BANKSEL	TRISA
	movlw		TRISA_VAL
	movwf		TRISA
	BANKSEL	ANSELA
	clrf		ANSELA						; no analog inputs

; Interrupts
	clrf		INTCON						; clear global interrupt enable
	BANKSEL	PIE1
	clrf		PIE1							; clear peripheral interrupt enables
	BANKSEL	PORTA
	return
; init

	END
